@@ -88,8 +88,6 @@ enum
   LAST_SIGNAL
 };
 
-static void gst_bus_class_init (GstBusClass * klass);
-static void gst_bus_init (GstBus * bus);
 static void gst_bus_dispose (GObject * object);
 
 static void gst_bus_set_main_context (GstBus * bus, GMainContext * ctx);
@@ -267,7 +265,7 @@ gst_bus_set_main_context (GstBus * bus, GMainContext * ctx)
  *
  * Creates a new #GstBus instance.
  *
- * Returns: a new #GstBus instance
+ * Returns: (transfer full): a new #GstBus instance
  */
 GstBus *
 gst_bus_new (void)
@@ -283,7 +281,7 @@ gst_bus_new (void)
 /**
  * gst_bus_post:
  * @bus: a #GstBus to post on
- * @message: The #GstMessage to post
+ * @message: (transfer full): the #GstMessage to post
  *
  * Post a message on the given bus. Ownership of the message
  * is taken by the bus.
@@ -470,10 +468,10 @@ gst_bus_set_flushing (GstBus * bus, gboolean flushing)
  * @timeout is #GST_CLOCK_TIME_NONE, this function will block forever until a
  * matching message was posted on the bus.
  *
- * Returns: a #GstMessage matching the filter in @types, or NULL if no matching
- * message was found on the bus until the timeout expired.
- * The message is taken from the bus and needs to be unreffed with
- * gst_message_unref() after usage.
+ * Returns: (transfer full): a #GstMessage matching the filter in @types,
+ *     or NULL if no matching message was found on the bus until the timeout
+ *     expired. The message is taken from the bus and needs to be unreffed
+ *     with gst_message_unref() after usage.
  *
  * MT safe.
  *
@@ -560,8 +558,8 @@ beach:
  * #GST_CLOCK_TIME_NONE, this function will block forever until a message was
  * posted on the bus.
  *
- * Returns: The #GstMessage that is on the bus after the specified timeout
- * or NULL if the bus is empty after the timeout expired.
+ * Returns: (transfer full): the #GstMessage that is on the bus after the
+ *     specified timeout or NULL if the bus is empty after the timeout expired.
  * The message is taken from the bus and needs to be unreffed with
  * gst_message_unref() after usage.
  *
@@ -587,10 +585,10 @@ gst_bus_timed_pop (GstBus * bus, GstClockTime timeout)
  * message that does match @type.  If there is no message matching @type on
  * the bus, all messages will be discarded.
  *
- * Returns: The next #GstMessage matching @type that is on the bus, or NULL if
- *     the bus is empty or there is no message matching @type.
- * The message is taken from the bus and needs to be unreffed with
- * gst_message_unref() after usage.
+ * Returns: (transfer full): the next #GstMessage matching @type that is on
+ *     the bus, or NULL if the bus is empty or there is no message matching
+ *     @type. The message is taken from the bus and needs to be unreffed with
+ *     gst_message_unref() after usage.
  *
  * MT safe.
  *
@@ -611,9 +609,9 @@ gst_bus_pop_filtered (GstBus * bus, GstMessageType types)
  *
  * Get a message from the bus.
  *
- * Returns: The #GstMessage that is on the bus, or NULL if the bus is empty.
- * The message is taken from the bus and needs to be unreffed with
- * gst_message_unref() after usage.
+ * Returns: (transfer full): the #GstMessage that is on the bus, or NULL if the
+ *     bus is empty. The message is taken from the bus and needs to be unreffed
+ *     with gst_message_unref() after usage.
  *
  * MT safe.
  */
@@ -633,7 +631,8 @@ gst_bus_pop (GstBus * bus)
  * on the bus' message queue. A reference is returned, and needs to be unreffed
  * by the caller.
  *
- * Returns: The #GstMessage that is on the bus, or NULL if the bus is empty.
+ * Returns: (transfer full): the #GstMessage that is on the bus, or NULL if the
+ *     bus is empty.
  *
  * MT safe.
  */
@@ -810,7 +809,7 @@ static GSourceFuncs gst_bus_source_funcs = {
  * a message is on the bus. After the GSource is dispatched, the
  * message is popped off the bus and unreffed.
  *
- * Returns: A #GSource that can be added to a mainloop.
+ * Returns: (transfer full): a #GSource that can be added to a mainloop.
  */
 GSource *
 gst_bus_create_watch (GstBus * bus)
@@ -867,7 +866,8 @@ gst_bus_add_watch_full_unlocked (GstBus * bus, gint priority,
  * @user_data: user data passed to @func.
  * @notify: the function to call when the source is removed.
  *
- * Adds a bus watch to the default main context with the given @priority.
+ * Adds a bus watch to the default main context with the given @priority (e.g.
+ * %G_PRIORITY_DEFAULT).
  * This function is used to receive asynchronous messages in the main loop.
  * There can only be a single bus watch per bus, you must remove it before you
  * can set a new one.
@@ -903,7 +903,8 @@ gst_bus_add_watch_full (GstBus * bus, gint priority,
  * @func: A function to call when a message is received.
  * @user_data: user data passed to @func.
  *
- * Adds a bus watch to the default main context with the default priority.
+ * Adds a bus watch to the default main context with the default priority
+ * (%G_PRIORITY_DEFAULT).
  * This function is used to receive asynchronous messages in the main loop.
  * There can only be a single bus watch per bus, you must remove it before you
  * can set a new one.
@@ -944,7 +945,7 @@ poll_func (GstBus * bus, GstMessage * message, GstBusPollData * poll_data)
   type = GST_MESSAGE_TYPE (message);
 
   if (type & poll_data->events) {
-    g_return_if_fail (poll_data->message == NULL);
+    g_assert (poll_data->message == NULL);
     /* keep ref to message */
     poll_data->message = gst_message_ref (message);
     GST_DEBUG ("mainloop %p quit", poll_data->loop);
@@ -971,7 +972,7 @@ poll_destroy (GstBusPollData * poll_data, gpointer unused)
   poll_data->source_running = FALSE;
   if (!poll_data->timeout_id) {
     g_main_loop_unref (poll_data->loop);
-    g_free (poll_data);
+    g_slice_free (GstBusPollData, poll_data);
   }
 }
 
@@ -981,7 +982,7 @@ poll_destroy_timeout (GstBusPollData * poll_data)
   poll_data->timeout_id = 0;
   if (!poll_data->source_running) {
     g_main_loop_unref (poll_data->loop);
-    g_free (poll_data);
+    g_slice_free (GstBusPollData, poll_data);
   }
 }
 
@@ -1025,9 +1026,9 @@ poll_destroy_timeout (GstBusPollData * poll_data)
  * better handled by setting up an asynchronous bus watch and doing things
  * from there.
  *
- * Returns: The message that was received, or NULL if the poll timed out.
- * The message is taken from the bus and needs to be unreffed with
- * gst_message_unref() after usage.
+ * Returns: (transfer full): the message that was received, or NULL if the
+ *     poll timed out. The message is taken from the bus and needs to be
+ *     unreffed with gst_message_unref() after usage.
  */
 GstMessage *
 gst_bus_poll (GstBus * bus, GstMessageType events, GstClockTimeDiff timeout)
@@ -1036,7 +1037,7 @@ gst_bus_poll (GstBus * bus, GstMessageType events, GstClockTimeDiff timeout)
   GstMessage *ret;
   gulong id;
 
-  poll_data = g_new0 (GstBusPollData, 1);
+  poll_data = g_slice_new (GstBusPollData);
   poll_data->source_running = TRUE;
   poll_data->loop = g_main_loop_new (NULL, FALSE);
   poll_data->events = events;
@@ -1193,7 +1194,8 @@ gst_bus_disable_sync_message_emission (GstBus * bus)
  * @bus: a #GstBus on which you want to receive the "message" signal
  * @priority: The priority of the watch.
  *
- * Adds a bus signal watch to the default main context with the given priority.
+ * Adds a bus signal watch to the default main context with the given @priority
+ * (e.g. %G_PRIORITY_DEFAULT).
  * After calling this statement, the bus will emit the "message" signal for each
  * message posted on the bus when the main loop is running.
  *
@@ -1247,8 +1249,8 @@ add_failed:
  * gst_bus_add_signal_watch:
  * @bus: a #GstBus on which you want to receive the "message" signal
  *
- * Adds a bus signal watch to the default main context with the default
- * priority.
+ * Adds a bus signal watch to the default main context with the default priority
+ * (%G_PRIORITY_DEFAULT).
  * After calling this statement, the bus will emit the "message" signal for each
  * message posted on the bus.
  *
