@@ -127,6 +127,10 @@ _gst_tag_initialize (void)
       _("The artist of the entire album, as it should be sorted"), NULL);
   gst_tag_register (GST_TAG_DATE, GST_TAG_FLAG_META, GST_TYPE_DATE,
       _("date"), _("date the data was created (as a GDate structure)"), NULL);
+  gst_tag_register (GST_TAG_DATE_TIME, GST_TAG_FLAG_META, GST_TYPE_DATE_TIME,
+      _("datetime"),
+      _("date and time the data was created (as a GstDateTime structure)"),
+      NULL);
   gst_tag_register (GST_TAG_GENRE, GST_TAG_FLAG_META,
       G_TYPE_STRING,
       _("genre"),
@@ -160,15 +164,13 @@ _gst_tag_initialize (void)
       gst_tag_merge_use_first);
   gst_tag_register (GST_TAG_LOCATION, GST_TAG_FLAG_META,
       G_TYPE_STRING,
-      _("location"),
-      _
-      ("Origin of media as a URI (location, where the original of the file or stream is hosted)"),
+      _("location"), _("Origin of media as a URI (location, where the "
+          "original of the file or stream is hosted)"),
       gst_tag_merge_strings_with_comma);
   gst_tag_register (GST_TAG_HOMEPAGE, GST_TAG_FLAG_META,
       G_TYPE_STRING,
       _("homepage"),
-      _
-      ("Homepage for this media (i.e. artist or movie homepage)"),
+      _("Homepage for this media (i.e. artist or movie homepage)"),
       gst_tag_merge_strings_with_comma);
   gst_tag_register (GST_TAG_DESCRIPTION, GST_TAG_FLAG_META, G_TYPE_STRING,
       _("description"), _("short text describing the content of the data"),
@@ -179,8 +181,9 @@ _gst_tag_initialize (void)
       _
       ("International Standard Recording Code - see http://www.ifpi.org/isrc/"),
       NULL);
-  gst_tag_register (GST_TAG_ORGANIZATION, GST_TAG_FLAG_META, G_TYPE_STRING, _("organization"), _("organization"),       /* FIXME */
-      gst_tag_merge_strings_with_comma);
+  /* FIXME: organization (fix what? tpm) */
+  gst_tag_register (GST_TAG_ORGANIZATION, GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("organization"), _("organization"), gst_tag_merge_strings_with_comma);
   gst_tag_register (GST_TAG_COPYRIGHT, GST_TAG_FLAG_META,
       G_TYPE_STRING, _("copyright"), _("copyright notice of the data"), NULL);
   gst_tag_register (GST_TAG_COPYRIGHT_URI, GST_TAG_FLAG_META,
@@ -256,6 +259,7 @@ _gst_tag_initialize (void)
   gst_tag_register (GST_TAG_IMAGE, GST_TAG_FLAG_META, GST_TYPE_BUFFER,
       _("image"), _("image related to this stream"), gst_tag_merge_use_first);
   gst_tag_register (GST_TAG_PREVIEW_IMAGE, GST_TAG_FLAG_META, GST_TYPE_BUFFER,
+      /* TRANSLATORS: 'preview image' = image that shows a preview of the full image */
       _("preview image"), _("preview image related to this stream"), NULL);
   gst_tag_register (GST_TAG_ATTACHMENT, GST_TAG_FLAG_META, GST_TYPE_BUFFER,
       _("attachment"), _("file attached to this stream"),
@@ -266,30 +270,106 @@ _gst_tag_initialize (void)
       _("keywords"), _("comma separated keywords describing the content"),
       gst_tag_merge_strings_with_comma);
   gst_tag_register (GST_TAG_GEO_LOCATION_NAME, GST_TAG_FLAG_META, G_TYPE_STRING,
-      _("geo location name"),
-      _
-      ("human readable descriptive location of where the media has been recorded or produced"),
-      NULL);
+      _("geo location name"), _("human readable descriptive location of where "
+          "the media has been recorded or produced"), NULL);
   gst_tag_register (GST_TAG_GEO_LOCATION_LATITUDE, GST_TAG_FLAG_META,
       G_TYPE_DOUBLE, _("geo location latitude"),
-      _
-      ("geo latitude location of where the media has been recorded or produced in degrees according to WGS84 (zero at the equator, negative values for southern latitudes)"),
-      NULL);
+      _("geo latitude location of where the media has been recorded or "
+          "produced in degrees according to WGS84 (zero at the equator, "
+          "negative values for southern latitudes)"), NULL);
   gst_tag_register (GST_TAG_GEO_LOCATION_LONGITUDE, GST_TAG_FLAG_META,
       G_TYPE_DOUBLE, _("geo location longitude"),
-      _
-      ("geo longitude location of where the media has been recorded or produced in degrees according to WGS84 (zero at the prime meridian in Greenwich/UK,  negative values for western longitudes)"),
-      NULL);
+      _("geo longitude location of where the media has been recorded or "
+          "produced in degrees according to WGS84 (zero at the prime meridian "
+          "in Greenwich/UK,  negative values for western longitudes)"), NULL);
   gst_tag_register (GST_TAG_GEO_LOCATION_ELEVATION, GST_TAG_FLAG_META,
       G_TYPE_DOUBLE, _("geo location elevation"),
-      _
-      ("geo elevation of where the media has been recorded or produced in meters according to WGS84 (zero is average sea level)"),
+      _("geo elevation of where the media has been recorded or produced in "
+          "meters according to WGS84 (zero is average sea level)"), NULL);
+  gst_tag_register (GST_TAG_GEO_LOCATION_COUNTRY, GST_TAG_FLAG_META,
+      G_TYPE_STRING, _("geo location country"),
+      _("country (english name) where the media has been recorded "
+          "or produced"), NULL);
+  gst_tag_register (GST_TAG_GEO_LOCATION_CITY, GST_TAG_FLAG_META,
+      G_TYPE_STRING, _("geo location city"),
+      _("city (english name) where the media has been recorded "
+          "or produced"), NULL);
+  gst_tag_register (GST_TAG_GEO_LOCATION_SUBLOCATION, GST_TAG_FLAG_META,
+      G_TYPE_STRING, _("geo location sublocation"),
+      _("a location whithin a city where the media has been produced "
+          "or created (e.g. the neighborhood)"), NULL);
+  gst_tag_register (GST_TAG_GEO_LOCATION_HORIZONTAL_ERROR, GST_TAG_FLAG_META,
+      G_TYPE_DOUBLE, _("geo location horizontal error"),
+      _("expected error of the horizontal positioning measures (in meters)"),
       NULL);
+  gst_tag_register (GST_TAG_GEO_LOCATION_MOVEMENT_SPEED, GST_TAG_FLAG_META,
+      G_TYPE_DOUBLE, _("geo location movement speed"),
+      _("movement speed of the capturing device while performing the capture "
+          "in m/s"), NULL);
+  gst_tag_register (GST_TAG_GEO_LOCATION_MOVEMENT_DIRECTION, GST_TAG_FLAG_META,
+      G_TYPE_DOUBLE, _("geo location movement direction"),
+      _("indicates the movement direction of the device performing the capture"
+          " of a media. It is represented as degrees in floating point "
+          "representation, 0 means the geographic north, and increases "
+          "clockwise"), NULL);
+  gst_tag_register (GST_TAG_GEO_LOCATION_CAPTURE_DIRECTION, GST_TAG_FLAG_META,
+      G_TYPE_DOUBLE, _("geo location capture direction"),
+      _("indicates the direction the device is pointing to when capturing "
+          " a media. It is represented as degrees in floating point "
+          " representation, 0 means the geographic north, and increases "
+          "clockwise"), NULL);
+  gst_tag_register (GST_TAG_SHOW_NAME, GST_TAG_FLAG_META, G_TYPE_STRING,
+      /* TRANSLATORS: 'show name' = 'TV/radio/podcast show name' here */
+      _("show name"),
+      _("Name of the tv/podcast/series show the media is from"),
+      gst_tag_merge_strings_with_comma);
+  gst_tag_register (GST_TAG_SHOW_SORTNAME, GST_TAG_FLAG_META, G_TYPE_STRING,
+      /* TRANSLATORS: 'show sortname' = 'TV/radio/podcast show name as used for sorting purposes' here */
+      _("show sortname"),
+      _("Name of the tv/podcast/series show the media is from, for sorting "
+          "purposes"), NULL);
+  gst_tag_register (GST_TAG_SHOW_EPISODE_NUMBER, GST_TAG_FLAG_META, G_TYPE_UINT,
+      _("episode number"),
+      _("The episode number in the season the media is part of"),
+      gst_tag_merge_use_first);
+  gst_tag_register (GST_TAG_SHOW_SEASON_NUMBER, GST_TAG_FLAG_META, G_TYPE_UINT,
+      _("season number"),
+      _("The season number of the show the media is part of"),
+      gst_tag_merge_use_first);
+  gst_tag_register (GST_TAG_LYRICS, GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("lyrics"), _("The lyrics of the media, commonly used for songs"),
+      gst_tag_merge_strings_with_comma);
+  gst_tag_register (GST_TAG_COMPOSER_SORTNAME, GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("composer sortname"),
+      _("person(s) who composed the recording, for sorting purposes"), NULL);
+  gst_tag_register (GST_TAG_GROUPING, GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("grouping"),
+      _("Groups related media that spans multiple tracks, like the different "
+          "pieces of a concerto. It is a higher level than a track, "
+          "but lower than an album"), NULL);
+  gst_tag_register (GST_TAG_USER_RATING, GST_TAG_FLAG_META, G_TYPE_UINT,
+      _("user rating"),
+      _("Rating attributed by a user. The higher the rank, "
+          "the more the user likes this media"), NULL);
+  gst_tag_register (GST_TAG_DEVICE_MANUFACTURER, GST_TAG_FLAG_META,
+      G_TYPE_STRING, _("device manufacturer"),
+      _("Manufacturer of the device used to create this media"), NULL);
+  gst_tag_register (GST_TAG_DEVICE_MODEL, GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("device model"),
+      _("Model of the device used to create this media"), NULL);
+  gst_tag_register (GST_TAG_APPLICATION_NAME, GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("application name"), _("Application used to create the media"), NULL);
+  gst_tag_register (GST_TAG_APPLICATION_DATA, GST_TAG_FLAG_META,
+      GST_TYPE_BUFFER, _("application data"),
+      _("Arbitrary application data to be serialized into the media"), NULL);
+  gst_tag_register (GST_TAG_IMAGE_ORIENTATION, GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("image orientation"),
+      _("How the image should be rotated or flipped before display"), NULL);
 }
 
 /**
  * gst_tag_merge_use_first:
- * @dest: uninitialized GValue to store result in
+ * @dest: (out caller-allocates): uninitialized GValue to store result in
  * @src: GValue to copy from
  *
  * This is a convenience function for the func argument of gst_tag_register().
@@ -306,7 +386,7 @@ gst_tag_merge_use_first (GValue * dest, const GValue * src)
 
 /**
  * gst_tag_merge_strings_with_comma:
- * @dest: uninitialized GValue to store result in
+ * @dest: (out caller-allocates): uninitialized GValue to store result in
  * @src: GValue to copy from
  *
  * This is a convenience function for the func argument of gst_tag_register().
@@ -317,17 +397,15 @@ void
 gst_tag_merge_strings_with_comma (GValue * dest, const GValue * src)
 {
   GString *str;
-
   gint i, count;
 
   count = gst_value_list_get_size (src);
   str = g_string_new (g_value_get_string (gst_value_list_get_value (src, 0)));
   for (i = 1; i < count; i++) {
-    /* seperator between two string */
-    str = g_string_append (str, _(", "));
-    str =
-        g_string_append (str, g_value_get_string (gst_value_list_get_value (src,
-                1)));
+    /* separator between two strings */
+    g_string_append (str, _(", "));
+    g_string_append (str,
+        g_value_get_string (gst_value_list_get_value (src, i)));
   }
 
   g_value_init (dest, G_TYPE_STRING);
@@ -376,14 +454,13 @@ gst_tag_lookup (GQuark entry)
  * merge function was supplied and if so which one.
  *
  * Two default merge functions are provided: gst_tag_merge_use_first() and
- * gst_tag_merge_strings_with_commas().
+ * gst_tag_merge_strings_with_comma().
  */
 void
 gst_tag_register (const gchar * name, GstTagFlag flag, GType type,
     const gchar * nick, const gchar * blurb, GstTagMergeFunc func)
 {
   GQuark key;
-
   GstTagInfo *info;
 
   g_return_if_fail (name != NULL);
@@ -399,7 +476,7 @@ gst_tag_register (const gchar * name, GstTagFlag flag, GType type,
     return;
   }
 
-  info = g_new (GstTagInfo, 1);
+  info = g_slice_new (GstTagInfo);
   info->flag = flag;
   info->type = type;
   info->nick = g_strdup (nick);
@@ -535,7 +612,9 @@ gst_tag_is_fixed (const gchar * tag)
  *
  * Creates a new empty GstTagList.
  *
- * Returns: An empty tag list
+ * Free-function: gst_tag_list_free
+ *
+ * Returns: (transfer full): An empty tag list
  */
 GstTagList *
 gst_tag_list_new (void)
@@ -556,8 +635,10 @@ gst_tag_list_new (void)
  * function. The tag list will make copies of any arguments passed
  * (e.g. strings, buffers).
  *
- * Returns: a new #GstTagList. Free with gst_tag_list_free() when no longer
- *     needed.
+ * Free-function: gst_tag_list_free
+ *
+ * Returns: (transfer full): a new #GstTagList. Free with gst_tag_list_free()
+ *     when no longer needed.
  *
  * Since: 0.10.24
  */
@@ -585,8 +666,10 @@ gst_tag_list_new_full (const gchar * tag, ...)
  * Just like gst_tag_list_new_full(), only that it takes a va_list argument.
  * Useful mostly for language bindings.
  *
- * Returns: a new #GstTagList. Free with gst_tag_list_free() when no longer
- *     needed.
+ * Free-function: gst_tag_list_free
+ *
+ * Returns: (transfer full): a new #GstTagList. Free with gst_tag_list_free()
+ *     when no longer needed.
  *
  * Since: 0.10.24
  */
@@ -650,13 +733,17 @@ GstTagCopyData;
 
 static void
 gst_tag_list_add_value_internal (GstStructure * list, GstTagMergeMode mode,
-    GQuark tag, const GValue * value)
+    GQuark tag, const GValue * value, GstTagInfo * info)
 {
-  GstTagInfo *info = gst_tag_lookup (tag);
-
   const GValue *value2;
 
-  g_assert (info != NULL);
+  if (info == NULL) {
+    info = gst_tag_lookup (tag);
+    if (G_UNLIKELY (info == NULL)) {
+      g_warning ("unknown tag '%s'", g_quark_to_string (tag));
+      return;
+    }
+  }
 
   if (info->merge_func
       && (value2 = gst_structure_id_get_value (list, tag)) != NULL) {
@@ -668,12 +755,12 @@ gst_tag_list_add_value_internal (GstStructure * list, GstTagMergeMode mode,
         gst_structure_id_set_value (list, tag, value);
         break;
       case GST_TAG_MERGE_PREPEND:
-        gst_value_list_concat (&dest, value, value2);
+        gst_value_list_merge (&dest, value, value2);
         gst_structure_id_set_value (list, tag, &dest);
         g_value_unset (&dest);
         break;
       case GST_TAG_MERGE_APPEND:
-        gst_value_list_concat (&dest, value2, value);
+        gst_value_list_merge (&dest, value2, value);
         gst_structure_id_set_value (list, tag, &dest);
         g_value_unset (&dest);
         break;
@@ -710,7 +797,7 @@ gst_tag_list_copy_foreach (GQuark tag, const GValue * value, gpointer user_data)
 {
   GstTagCopyData *copy = (GstTagCopyData *) user_data;
 
-  gst_tag_list_add_value_internal (copy->list, copy->mode, tag, value);
+  gst_tag_list_add_value_internal (copy->list, copy->mode, tag, value, NULL);
 
   return TRUE;
 }
@@ -748,7 +835,9 @@ gst_tag_list_insert (GstTagList * into, const GstTagList * from,
  *
  * Copies a given #GstTagList.
  *
- * Returns: copy of the given list
+ * Free-function: gst_tag_list_free
+ *
+ * Returns: (transfer full): copy of the given list
  */
 GstTagList *
 gst_tag_list_copy (const GstTagList * list)
@@ -767,7 +856,9 @@ gst_tag_list_copy (const GstTagList * list)
  * Merges the two given lists into a new list. If one of the lists is NULL, a
  * copy of the other is returned. If both lists are NULL, NULL is returned.
  *
- * Returns: the new list
+ * Free-function: gst_tag_list_free
+ *
+ * Returns: (transfer full): the new list
  */
 GstTagList *
 gst_tag_list_merge (const GstTagList * list1, const GstTagList * list2,
@@ -799,7 +890,7 @@ gst_tag_list_merge (const GstTagList * list1, const GstTagList * list2,
 
 /**
  * gst_tag_list_free:
- * @list: the list to free
+ * @list: (in) (transfer full): the list to free
  *
  * Frees the given list and all associated values.
  */
@@ -913,11 +1004,16 @@ gst_tag_list_add_valist (GstTagList * list, GstTagMergeMode mode,
 
     quark = g_quark_from_string (tag);
     info = gst_tag_lookup (quark);
-    if (info == NULL)
-      g_warning ("no GstTag for %s", tag);
-    g_return_if_fail (info != NULL);
+    if (G_UNLIKELY (info == NULL)) {
+      g_warning ("unknown tag '%s'", tag);
+      return;
+    }
+#if GLIB_CHECK_VERSION(2,23,3)
+    G_VALUE_COLLECT_INIT (&value, info->type, var_args, 0, &error);
+#else
     g_value_init (&value, info->type);
     G_VALUE_COLLECT (&value, var_args, 0, &error);
+#endif
     if (error) {
       g_warning ("%s: %s", G_STRLOC, error);
       g_free (error);
@@ -926,7 +1022,7 @@ gst_tag_list_add_valist (GstTagList * list, GstTagMergeMode mode,
        */
       return;
     }
-    gst_tag_list_add_value_internal (list, mode, quark, &value);
+    gst_tag_list_add_value_internal (list, mode, quark, &value, info);
     g_value_unset (&value);
     tag = va_arg (var_args, gchar *);
   }
@@ -959,7 +1055,7 @@ gst_tag_list_add_valist_values (GstTagList * list, GstTagMergeMode mode,
     quark = g_quark_from_string (tag);
     g_return_if_fail (gst_tag_lookup (quark) != NULL);
     gst_tag_list_add_value_internal (list, mode, quark, va_arg (var_args,
-            GValue *));
+            GValue *), NULL);
     tag = va_arg (var_args, gchar *);
   }
 }
@@ -984,7 +1080,7 @@ gst_tag_list_add_value (GstTagList * list, GstTagMergeMode mode,
   g_return_if_fail (tag != NULL);
 
   gst_tag_list_add_value_internal (list, mode, g_quark_from_string (tag),
-      value);
+      value, NULL);
 }
 
 /**
@@ -1024,8 +1120,8 @@ structure_foreach_wrapper (GQuark field_id, const GValue * value,
 /**
  * gst_tag_list_foreach:
  * @list: list to iterate over
- * @func: function to be called for each tag
- * @user_data: user specified data
+ * @func: (scope call): function to be called for each tag
+ * @user_data: (closure): user specified data
  *
  * Calls the given function for each tag inside the tag list. Note that if there
  * is no tag, the function won't be called at all.
@@ -1055,8 +1151,8 @@ gst_tag_list_foreach (const GstTagList * list, GstTagForeachFunc func,
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: The GValue for the specified entry or NULL if the tag wasn't
- *          available or the tag doesn't have as many entries
+ * Returns: (transfer none): The GValue for the specified entry or NULL if the
+ *          tag wasn't available or the tag doesn't have as many entries
  */
 G_CONST_RETURN GValue *
 gst_tag_list_get_value_index (const GstTagList * list, const gchar * tag,
@@ -1084,7 +1180,7 @@ gst_tag_list_get_value_index (const GstTagList * list, const gchar * tag,
 
 /**
  * gst_tag_list_copy_value:
- * @dest: uninitialized #GValue to copy into
+ * @dest: (out caller-allocates): uninitialized #GValue to copy into
  * @list: list to get the tag from
  * @tag: tag to read out
  *
@@ -1182,7 +1278,7 @@ gst_tag_list_get_ ## name ## _index (const GstTagList *list,            \
  * gst_tag_list_get_char:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1195,7 +1291,7 @@ gst_tag_list_get_ ## name ## _index (const GstTagList *list,            \
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1203,12 +1299,12 @@ gst_tag_list_get_ ## name ## _index (const GstTagList *list,            \
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (char, gchar, TRUE)
+TAG_MERGE_FUNCS (char, gchar, TRUE);
 /**
  * gst_tag_list_get_uchar:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1221,7 +1317,7 @@ TAG_MERGE_FUNCS (char, gchar, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1229,12 +1325,12 @@ TAG_MERGE_FUNCS (char, gchar, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (uchar, guchar, TRUE)
+TAG_MERGE_FUNCS (uchar, guchar, TRUE);
 /**
  * gst_tag_list_get_boolean:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1247,7 +1343,7 @@ TAG_MERGE_FUNCS (uchar, guchar, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1255,12 +1351,12 @@ TAG_MERGE_FUNCS (uchar, guchar, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (boolean, gboolean, TRUE)
+TAG_MERGE_FUNCS (boolean, gboolean, TRUE);
 /**
  * gst_tag_list_get_int:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1273,7 +1369,7 @@ TAG_MERGE_FUNCS (boolean, gboolean, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1281,12 +1377,12 @@ TAG_MERGE_FUNCS (boolean, gboolean, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (int, gint, TRUE)
+TAG_MERGE_FUNCS (int, gint, TRUE);
 /**
  * gst_tag_list_get_uint:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1299,7 +1395,7 @@ TAG_MERGE_FUNCS (int, gint, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1307,12 +1403,12 @@ TAG_MERGE_FUNCS (int, gint, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (uint, guint, TRUE)
+TAG_MERGE_FUNCS (uint, guint, TRUE);
 /**
  * gst_tag_list_get_long:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1325,7 +1421,7 @@ TAG_MERGE_FUNCS (uint, guint, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1333,12 +1429,12 @@ TAG_MERGE_FUNCS (uint, guint, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (long, glong, TRUE)
+TAG_MERGE_FUNCS (long, glong, TRUE);
 /**
  * gst_tag_list_get_ulong:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1351,7 +1447,7 @@ TAG_MERGE_FUNCS (long, glong, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1359,12 +1455,12 @@ TAG_MERGE_FUNCS (long, glong, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (ulong, gulong, TRUE)
+TAG_MERGE_FUNCS (ulong, gulong, TRUE);
 /**
  * gst_tag_list_get_int64:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1377,7 +1473,7 @@ TAG_MERGE_FUNCS (ulong, gulong, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1385,12 +1481,12 @@ TAG_MERGE_FUNCS (ulong, gulong, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (int64, gint64, TRUE)
+TAG_MERGE_FUNCS (int64, gint64, TRUE);
 /**
  * gst_tag_list_get_uint64:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1403,7 +1499,7 @@ TAG_MERGE_FUNCS (int64, gint64, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1411,12 +1507,12 @@ TAG_MERGE_FUNCS (int64, gint64, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (uint64, guint64, TRUE)
+TAG_MERGE_FUNCS (uint64, guint64, TRUE);
 /**
  * gst_tag_list_get_float:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1429,7 +1525,7 @@ TAG_MERGE_FUNCS (uint64, guint64, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1437,12 +1533,12 @@ TAG_MERGE_FUNCS (uint64, guint64, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (float, gfloat, TRUE)
+TAG_MERGE_FUNCS (float, gfloat, TRUE);
 /**
  * gst_tag_list_get_double:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1455,7 +1551,7 @@ TAG_MERGE_FUNCS (float, gfloat, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1463,12 +1559,12 @@ TAG_MERGE_FUNCS (float, gfloat, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (double, gdouble, TRUE)
+TAG_MERGE_FUNCS (double, gdouble, TRUE);
 /**
  * gst_tag_list_get_pointer:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out) (transfer none): location for the result
  *
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
@@ -1481,7 +1577,7 @@ TAG_MERGE_FUNCS (double, gdouble, TRUE)
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out) (transfer none): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1489,14 +1585,25 @@ TAG_MERGE_FUNCS (double, gdouble, TRUE)
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (pointer, gpointer, (*value != NULL))
+TAG_MERGE_FUNCS (pointer, gpointer, (*value != NULL));
+
+static inline gchar *
+_gst_strdup0 (const gchar * s)
+{
+  if (s == NULL || *s == '\0')
+    return NULL;
+
+  return g_strdup (s);
+}
+
 #undef COPY_FUNC
-#define COPY_FUNC g_strdup
+#define COPY_FUNC _gst_strdup0
+
 /**
  * gst_tag_list_get_string:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: location for the result
+ * @value: (out callee-allocates) (transfer full): location for the result
  *
  * Copies the contents for the given tag into the value, possibly merging
  * multiple values into one if multiple values are associated with the tag.
@@ -1508,6 +1615,8 @@ TAG_MERGE_FUNCS (pointer, gpointer, (*value != NULL))
  * freed by the caller using g_free when no longer needed. Since 0.10.24 the
  * returned string is also guaranteed to be non-NULL and non-empty.
  *
+ * Free-function: g_free
+ *
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
@@ -1516,7 +1625,7 @@ TAG_MERGE_FUNCS (pointer, gpointer, (*value != NULL))
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out callee-allocates) (transfer full): location for the result
  *
  * Gets the value that is at the given index for the given tag in the given
  * list.
@@ -1525,21 +1634,64 @@ TAG_MERGE_FUNCS (pointer, gpointer, (*value != NULL))
  * freed by the caller using g_free when no longer needed. Since 0.10.24 the
  * returned string is also guaranteed to be non-NULL and non-empty.
  *
+ * Free-function: g_free
+ *
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list.
  */
-TAG_MERGE_FUNCS (string, gchar *, (*value != NULL && **value != '\0'))
+TAG_MERGE_FUNCS (string, gchar *, (*value != NULL));
+
+/*
+ *FIXME 0.11: Instead of _peek (non-copy) and _get (copy), we could have
+ *            _get (non-copy) and _dup (copy) for strings, seems more
+ *            widely used
+ */
+/**
+ * gst_tag_list_peek_string_index:
+ * @list: a #GstTagList to get the tag from
+ * @tag: tag to read out
+ * @index: number of entry to read out
+ * @value: (out) (transfer none): location for the result
+ *
+ * Peeks at the value that is at the given index for the given tag in the given
+ * list.
+ *
+ * The resulting string in @value will be in UTF-8 encoding and doesn't need
+ * to be freed by the caller. The returned string is also guaranteed to
+ * be non-NULL and non-empty.
+ *
+ * Returns: TRUE, if a value was set, FALSE if the tag didn't exist in the
+ *              given list.
+ */
+gboolean
+gst_tag_list_peek_string_index (const GstTagList * list,
+    const gchar * tag, guint index, const gchar ** value)
+{
+  const GValue *v;
+
+  g_return_val_if_fail (GST_IS_TAG_LIST (list), FALSE);
+  g_return_val_if_fail (tag != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+
+  if ((v = gst_tag_list_get_value_index (list, tag, index)) == NULL)
+    return FALSE;
+  *value = g_value_get_string (v);
+  return *value != NULL && **value != '\0';
+}
 
 /**
  * gst_tag_list_get_date:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: address of a GDate pointer variable to store the result into
+ * @value: (out callee-allocates) (transfer full): address of a GDate pointer
+ *     variable to store the result into
  *
  * Copies the first date for the given tag in the taglist into the variable
  * pointed to by @value. Free the date with g_date_free() when it is no longer
  * needed.
  *
+ * Free-function: g_date_free
+ *
  * Returns: TRUE, if a date was copied, FALSE if the tag didn't exist in the
  *              given list or if it was #NULL.
  */
@@ -1565,12 +1717,14 @@ gst_tag_list_get_date (const GstTagList * list, const gchar * tag,
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: location for the result
+ * @value: (out callee-allocates) (transfer full): location for the result
  *
  * Gets the date that is at the given index for the given tag in the given
  * list and copies it into the variable pointed to by @value. Free the date
  * with g_date_free() when it is no longer needed.
  *
+ * Free-function: g_date_free
+ *
  * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
  *              given list or if it was #NULL.
  */
@@ -1590,16 +1744,91 @@ gst_tag_list_get_date_index (const GstTagList * list,
   return (*value != NULL);
 }
 
+/**
+ * gst_tag_list_get_date_time:
+ * @list: a #GstTagList to get the tag from
+ * @tag: tag to read out
+ * @value: (out callee-allocates) (transfer full): address of a #GstDateTime
+ *     pointer variable to store the result into
+ *
+ * Copies the first datetime for the given tag in the taglist into the variable
+ * pointed to by @value. Unref the date with gst_date_time_unref() when
+ * it is no longer needed.
+ *
+ * Free-function: gst_date_time_unref
+ *
+ * Returns: TRUE, if a datetime was copied, FALSE if the tag didn't exist in
+ *              thegiven list or if it was #NULL.
+ *
+ * Since: 0.10.31
+ */
+gboolean
+gst_tag_list_get_date_time (const GstTagList * list, const gchar * tag,
+    GstDateTime ** value)
+{
+  GValue v = { 0, };
+
+  g_return_val_if_fail (GST_IS_TAG_LIST (list), FALSE);
+  g_return_val_if_fail (tag != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+
+  if (!gst_tag_list_copy_value (&v, list, tag))
+    return FALSE;
+
+  g_return_val_if_fail (GST_VALUE_HOLDS_DATE_TIME (&v), FALSE);
+
+  *value = (GstDateTime *) g_value_dup_boxed (&v);
+  g_value_unset (&v);
+  return (*value != NULL);
+}
+
+/**
+ * gst_tag_list_get_date_time_index:
+ * @list: a #GstTagList to get the tag from
+ * @tag: tag to read out
+ * @index: number of entry to read out
+ * @value: (out callee-allocates) (transfer full): location for the result
+ *
+ * Gets the datetime that is at the given index for the given tag in the given
+ * list and copies it into the variable pointed to by @value. Unref the datetime
+ * with gst_date_time_unref() when it is no longer needed.
+ *
+ * Free-function: gst_date_time_unref
+ *
+ * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ *              given list or if it was #NULL.
+ *
+ * Since: 0.10.31
+ */
+gboolean
+gst_tag_list_get_date_time_index (const GstTagList * list,
+    const gchar * tag, guint index, GstDateTime ** value)
+{
+  const GValue *v;
+
+  g_return_val_if_fail (GST_IS_TAG_LIST (list), FALSE);
+  g_return_val_if_fail (tag != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+
+  if ((v = gst_tag_list_get_value_index (list, tag, index)) == NULL)
+    return FALSE;
+  *value = (GstDateTime *) g_value_dup_boxed (v);
+  return (*value != NULL);
+}
+
 /**
  * gst_tag_list_get_buffer:
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
- * @value: address of a GstBuffer pointer variable to store the result into
+ * @value: (out callee-allocates) (transfer full): address of a GstBuffer
+ *     pointer variable to store the result into
  *
  * Copies the first buffer for the given tag in the taglist into the variable
  * pointed to by @value. Free the buffer with gst_buffer_unref() when it is
  * no longer needed.
  *
+ * Free-function: gst_buffer_unref
+ *
  * Returns: TRUE, if a buffer was copied, FALSE if the tag didn't exist in the
  *              given list or if it was #NULL.
  *
@@ -1627,12 +1856,15 @@ gst_tag_list_get_buffer (const GstTagList * list, const gchar * tag,
  * @list: a #GstTagList to get the tag from
  * @tag: tag to read out
  * @index: number of entry to read out
- * @value: address of a GstBuffer pointer variable to store the result into
+ * @value: (out callee-allocates) (transfer full): address of a GstBuffer
+ *     pointer variable to store the result into
  *
  * Gets the buffer that is at the given index for the given tag in the given
  * list and copies it into the variable pointed to by @value. Free the buffer
  * with gst_buffer_unref() when it is no longer needed.
  *
+ * Free-function: gst_buffer_unref
+ *
  * Returns: TRUE, if a buffer was copied, FALSE if the tag didn't exist in the
  *              given list or if it was #NULL.
  *
