@@ -1,5 +1,5 @@
 
-/* Generated data (by glib-mkenums) */
+
 
 #include "gst_private.h"
 #include <gst/gst.h>
@@ -184,6 +184,7 @@ gst_clock_return_get_type (void)
     {C_ENUM (GST_CLOCK_BADTIME), "GST_CLOCK_BADTIME", "badtime"},
     {C_ENUM (GST_CLOCK_ERROR), "GST_CLOCK_ERROR", "error"},
     {C_ENUM (GST_CLOCK_UNSUPPORTED), "GST_CLOCK_UNSUPPORTED", "unsupported"},
+    {C_ENUM (GST_CLOCK_DONE), "GST_CLOCK_DONE", "done"},
     {0, NULL, NULL}
   };
 
@@ -349,6 +350,7 @@ gst_element_flags_get_type (void)
     {C_FLAGS (GST_ELEMENT_IS_SINK), "GST_ELEMENT_IS_SINK", "is-sink"},
     {C_FLAGS (GST_ELEMENT_UNPARENTING), "GST_ELEMENT_UNPARENTING",
         "unparenting"},
+    {C_FLAGS (GST_ELEMENT_IS_SOURCE), "GST_ELEMENT_IS_SOURCE", "is-source"},
     {C_FLAGS (GST_ELEMENT_FLAG_LAST), "GST_ELEMENT_FLAG_LAST", "flag-last"},
     {0, NULL, NULL}
   };
@@ -1530,5 +1532,3 @@ gst_parse_flags_get_type (void)
 
   return (GType) id;
 }
-
-/* Generated data ends here */
